@@ -28,6 +28,7 @@ class BluetoothDS4Device(DS4Device):
         try:
             ctl_socket.connect((addr, L2CAP_PSM_HIDP_CTRL))
             int_socket.connect((addr, L2CAP_PSM_HIDP_INTR))
+            int_socket.setblocking(False)
         except socket.error as err:
             DeviceError("Failed to connect: {0}".format(err))
 
@@ -37,12 +38,16 @@ class BluetoothDS4Device(DS4Device):
         self.buf = bytearray(REPORT_SIZE)
         self.ctl_sock = ctl_sock
         self.int_sock = int_sock
+        self.report_fd = int_sock.fileno()
 
         super(BluetoothDS4Device, self).__init__(addr.upper(), addr,
                                                  "bluetooth")
 
     def read_report(self):
-        ret = self.int_sock.recv_into(self.buf)
+        try:
+            ret = self.int_sock.recv_into(self.buf)
+        except IOError:
+            return
 
         # Disconnection
         if ret == 0:
@@ -103,7 +108,7 @@ class BluetoothBackend(Backend):
         devices = []
         res = res.splitlines()[1:]
         for _, bdaddr, name in map(lambda l: l.split(b"\t"), res):
-            devices.append((bdaddr.decode("ascii"), name.decode("ascii")))
+            devices.append((bdaddr.decode("utf8"), name.decode("utf8")))
 
         return devices
 
