@@ -1,9 +1,12 @@
 import fcntl
 import itertools
+import os
+
+from io import FileIO
+from time import sleep
 
 from evdev import InputDevice
 from pyudev import Context, Monitor
-from time import sleep
 
 from ..backend import Backend
 from ..exceptions import DeviceError
@@ -19,7 +22,8 @@ HIDIOCGFEATURE = lambda size: IOC_RW | (0x07 << 0) | (size << 16)
 class HidrawDS4Device(DS4Device):
     def __init__(self, name, addr, type, hidraw_device, event_device):
         try:
-            self.fd = open(hidraw_device, "rb+", 0)
+            self.report_fd = os.open(hidraw_device, os.O_RDWR | os.O_NONBLOCK)
+            self.fd = FileIO(self.report_fd, "rb+", closefd=False)
             self.input_device = InputDevice(event_device)
             self.input_device.grab()
         except (OSError, IOError) as err:
@@ -30,7 +34,10 @@ class HidrawDS4Device(DS4Device):
         super(HidrawDS4Device, self).__init__(name, addr, type)
 
     def read_report(self):
-        ret = self.fd.readinto(self.buf)
+        try:
+            ret = self.fd.readinto(self.buf)
+        except IOError:
+            return
 
         # Disconnection
         if ret == 0:
@@ -40,7 +47,11 @@ class HidrawDS4Device(DS4Device):
         if ret < self.report_size:
             return False
 
-        buf = self.get_trimmed_report_data()
+        if self.type == "bluetooth":
+            # Cut off bluetooth data
+            buf = zero_copy_slice(self.buf, 2)
+        else:
+            buf = self.buf
 
         return self.parse_report(buf)
 
@@ -51,9 +62,6 @@ class HidrawDS4Device(DS4Device):
 
         return fcntl.ioctl(self.fd, op, bytes(buf))
 
-    def get_trimmed_report_data(self):
-        raise NotImplementedError
-
     def write_report(self, report_id, data):
         if self.type == "bluetooth":
             # TODO: Add a check for a kernel that supports writing
@@ -70,34 +78,23 @@ class HidrawDS4Device(DS4Device):
         except IOError:
             pass
 
-    @property
-    def report_size(self):
-        raise NotImplementedError
-
 
 class HidrawBluetoothDS4Device(HidrawDS4Device):
     __type__ = "bluetooth"
 
-    def get_trimmed_report_data(self):
-        # Cut off bluetooth data
-        return zero_copy_slice(self.buf, 2)
+    report_size = 78
 
     def set_operational(self):
         self.read_feature_report(0x02, 37)
 
-    @property
-    def report_size(self):
-        return 78
-
 
 class HidrawUSBDS4Device(HidrawDS4Device):
     __type__ = "usb"
 
-    def get_trimmed_report_data(self):
-        return self.buf
+    report_size = 64
 
     def set_operational(self):
-        # Get the bluetooth MAC and set operational with a single report
+        # Get the bluetooth MAC
         addr = self.read_feature_report(0x81, 6)[1:]
         addr = ["{0:02x}".format(c) for c in bytearray(addr)]
         addr = ":".join(reversed(addr)).upper()
@@ -105,10 +102,6 @@ class HidrawUSBDS4Device(HidrawDS4Device):
         self.device_name = "{0} {1}".format(addr, self.device_name)
         self.device_addr = addr
 
-    @property
-    def report_size(self):
-        return 64
-
 
 HID_DEVICES = {
     "Sony Computer Entertainment Wireless Controller": HidrawUSBDS4Device,
