@@ -3,10 +3,26 @@ import sys
 from .device import DS4Report
 
 
-VALID_BUTTONS = DS4Report._fields
+VALID_BUTTONS = DS4Report.__slots__
 
 
-def parse_button_combo(combo):
+def iter_except(func, exception, first=None):
+    """Call a function repeatedly until an exception is raised.
+
+    Converts a call-until-exception interface to an iterator interface.
+    Like __builtin__.iter(func, sentinel) but uses an exception instead
+    of a sentinel to end the loop.
+    """
+    try:
+        if first is not None:
+            yield first()
+        while True:
+            yield func()
+    except exception:
+        pass
+
+
+def parse_button_combo(combo, sep="+"):
     def button_prefix(button):
         button = button.strip()
         if button in ("up", "down", "left", "right"):
@@ -19,7 +35,7 @@ def parse_button_combo(combo):
 
         return prefix + button
 
-    return tuple(map(button_prefix, combo.lower().split("+")))
+    return tuple(map(button_prefix, combo.lower().split(sep)))
 
 
 def zero_copy_slice(buf, start=None, end=None):
